
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../src/initng.h"

#include <time.h>                           /* time() */
#include <fcntl.h>                          /* fcntl() */
#include <sys/un.h>                         /* memmove() strcmp() */
#include <sys/wait.h>                       /* waitpid() sa */
#include <linux/kd.h>                       /* KDSIGACCEPT */
#include <sys/ioctl.h>                      /* ioctl() */
#include <stdlib.h>                         /* free() exit() */
#include <sys/reboot.h>                     /* reboot() RB_DISABLE_CAD */
#include <termios.h>
#include <stdio.h>
#include <sys/klog.h>
#include <dirent.h>
#include <sys/stat.h>
#include <fnmatch.h>
#include <sys/time.h>
#include <sys/resource.h>

#include "../src/initng_global.h"
#include "../src/main.h"
#include "../src/initng_signal.h"
#include "../src/initng_handler.h"
#include "../src/initng_execute.h"          /* new_environ() */
#include "../src/initng_active_db.h"
#include "../src/initng_load_module.h"
#include "../src/initng_plugin_callers.h"
#include "../src/initng_toolbox.h"
#include "../src/initng_main.h"
#include "../src/initng_service_data_types.h"
#include "../src/initng_fd.h"
#include "../src/initng_common.h"

/* include some soruce files directly, remember this is only a test */
#include "../src/initng_global.c"
#include "../plugins/stcmd/print_service.c"

/*
 * A test, that will load service, set by argument 1, and then exit
 */

#define SPACE for(i=0; i<level; i++, printf("  "))

static int load_service(char *name, int level);
static void parse_all(const char *dirname);
static int verbose = 0;

s_entry STATUS = { "status", INT, 6, NULL, "Status of testing." };

int main(int argc, char *argv[], char *env[])
{

    int parse_all_files = 0;

    /* initialise global variables */
    initng_new(argc, argv, env);

    if (!initng_load_all_modules())
    {
        printf("could not load all modules\n");
        goto exit;
    }

    /* enable verbose mode */
#ifdef DEBUG
    g.verbose = 0;
#endif

    int i;
    char *srv_name = i_strdup("default");
    
    for (i=1; i<argc; i++)
    {
	int opt_type = 0;
	char *opt = argv[i];

	if (strlen(opt)<=1)
	    continue;

	if (opt[0]=='-' && opt[1]=='-')
	{
	    if (strlen(opt)==2)
		continue;

	    opt_type = 1;
	    opt += 2;
	} 
	else if (opt[0]=='-')
	{
	    opt_type = 2;
	    opt++;
	}

	if (opt_type == 1)
	{
	    if (strcmp(opt, "verbose") == 0)
		verbose = 1;

	    if (strcmp(opt, "all") == 0)
		parse_all_files = 1;
	} 
	else if (opt_type == 2)
	{
	    if (strcmp(opt, "v") == 0)
		verbose = 1;

	    if (strcmp(opt, "a") == 0)
		parse_all_files = 1;
	} else
	{
	    free(srv_name);
	    srv_name = opt;
	}
    }

    if (parse_all_files == 1)
    {
        parse_all("system");
        parse_all("daemon");
        parse_all("net");
	//parse_all("debug");
    	parse_all("daemon/bluetooth");
    }
    else
    {
        load_service(srv_name,0);
    }

  exit:
    /* unload all modules */
    initng_unload_all_modules();
    service_db_free_all();
    initng_free();
    return (0);
}

static void parse_all(const char *dirname)
{
    struct dirent *e;
    char tmp[200];
    char tmp2[200];
    DIR *d;
    int i;

    sprintf(tmp, "/etc/initng/%s", dirname);

    d = opendir(tmp);
    if (!d)
        return;

    while ((e = readdir(d)))
    {
        if (e->d_name[0] == '.')
            continue;

        /* get string lenght */
        i = strlen(e->d_name);

        /* Check that is it a .i file */
        if (e->d_name[i - 1] != 'i' && e->d_name[i - 2] != '.')
            continue;

        strncpy(tmp, e->d_name, i - 2);
        tmp[i - 2] = '\0';
        sprintf(tmp2, "%s/%s", dirname, tmp);
    	load_service(tmp2,0);
    }

    closedir(d);
}

static int load_service(char *name, int level)
{
    service_h *service = NULL;
    service_h *tmp_service = NULL;
    char *string=NULL;
    int result = 0;
    int i;

    SPACE;
    printf("Probeparsing: %s\n", name);

    service = parse_service(name);
    if (!service)
    {
        /* unload all modules 
        initng_unload_all_modules();
        initng_free();
        exit(2);
	*/
	SPACE;
	printf("%s - failed\n",name);
	service = service_db_new(name, &TYPE_SERVICE);
	if (service)
	{
	    service_db_add(service);
	    service_db_set_int(&STATUS, service, -1);
	}
	return result;
    }

    if (verbose == 1)
    {
	printf("\n");
//	service_db_print_u(service, stdout, 1);
	service_db_print_all(stdout);
    }

    result = 1;

    while ((string = service_db_get_next_string(&NEED, service, string)))
    {
	tmp_service = service_db_find_by_name(string);
	if (!tmp_service || service_db_get_int(&STATUS, tmp_service)<=0)
	    result &= load_service(string, level+1);
    }

    service_db_set_int(&STATUS, service, 1);

    SPACE;
    if (result==0)
	printf("%s - failed\n",name);
    else
	printf("%s - ok\n",name);

    return result;
}
