/* Initng, a next generatind sysvinit replacement.
 * Copyright (C) 2005 neuron <neuron@hollowtube.mine.nu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"


#include <stdio.h>
#include <stdlib.h>                         /* free() exit() */
#include <string.h>
#include <assert.h>

#include "../../src/initng_handler.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_common.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_static_data_id.h"
#include "../../src/initng_static_states.h"

#include "initng_depend.h"

/* standard dep check */
static int dep_on(active_h * service, active_h * check)
{
    char *string = NULL;

    assert(service);
    assert(check);

    while ((string = active_db_get_next_string(&REQUIRE, service, string)))
    {
        if (strcmp(string, check->name) == 0)
            return (TRUE);
    }
    string = NULL;
    while ((string = active_db_get_next_string(&NEED, service, string)))
    {
        if (strcmp(string, check->name) == 0)
            return (TRUE);
    }
    string = NULL;
    while ((string = active_db_get_next_string(&USE, service, string)))
    {
        if (strcmp(string, check->name) == 0)
            return (TRUE);
    }
    return (FALSE);
}


int service_state(active_h * service)
{
    char *tmp = NULL;

    assert(service);
    assert(service->name);


    if (IS_MARK(service, &LOADING))
    {
        while ((tmp = active_db_get_next_string(&REQUIRE, service, tmp)))
        {
            /* look if it exits allredy */
            if (active_db_find_by_name(tmp))
            {
                D_("No need to LOAD %s, it is allredy loaded!\n", tmp);
                continue;
            }

            /* OKAY, start the service */

            if (!start_new_service_named(tmp))
            {
                F_("%s required dep \"%s\" could not start!\n", service->name,
                   tmp);
                stop_service(service);
                return (FAIL);
            }
        }

        while ((tmp = active_db_get_next_string(&NEED, service, tmp)))
        {
            /* look if it exits allredy */
            if (active_db_find_by_name(tmp))
            {
                D_("No need to LOAD %s, it is allredy loaded!\n", tmp);
                continue;
            }

            /* OKAY, start the service */

            if (!start_new_service_named(tmp))
            {
                W_("service \"%s\" needs service \"%s\", that could not be found!\n", service->name, tmp);
            }
        }
    }

    return (TRUE);
}



int module_init(const char *version)
{
    D_("module_init();\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    initng_add_hook(ASTATUS_CHANGE, 50, &service_state);
    initng_add_hook(DEP_ON, 50, &dep_on);
    return (TRUE);
}

void module_unload(void)
{
    D_("module_unload();\n");
    initng_del_hook(ASTATUS_CHANGE, &service_state);
    initng_del_hook(DEP_ON, &dep_on);
}
