
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "../../src/initng.h"


#include <stdio.h>
#include <string.h>
#include "../../src/initng_handler.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"


#include "initng_suid.h"
#include <assert.h>
#include <pwd.h>
#include <grp.h>

s_entry SUID = { "suid", STRING, 4, NULL, "Set this user id on launch." };
s_entry SGID = { "sgid", STRING, 4, NULL, "Set this group id on launch." };
int do_suid(active_h * s, process_h * p __attribute__ ((unused)))
{
    char *tmp = NULL;
    int ret = TRUE;

    assert(s);
    assert(s->name);
    assert(p);

    D_("do_suid!\n");

    if ((tmp = active_db_get_string(&SGID, s)))
    {
        struct group *gr = getgrnam(tmp);

        if (!gr)
        {
            F_("%s: GROUP \"%s\" not found!\n", s->name, tmp);
            ret = FALSE;
        }
        D_("setgid to: %i.\n", gr->gr_gid);
        setgid(gr->gr_gid);
    }

    if ((tmp = active_db_get_string(&SUID, s)))
    {
        struct passwd *pw = getpwnam(tmp);

        if (!pw)
        {
            F_("%s: USER \"%s\" not found!\n", s->name, tmp);
            ret = FALSE;
        }
        D_("setuid to: %i.\n", pw->pw_uid);
        setuid(pw->pw_uid);
    }
    return ret;
}

int module_init(const char *version)
{
    D_("module_init();\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    initng_sdt_add(&SUID);
    initng_sdt_add(&SGID);
    return (initng_add_hook(A_FORK, 90, &do_suid));
}

void module_unload(void)
{
    D_("module_unload();\n");
    initng_sdt_del(&SUID);
    initng_sdt_del(&SGID);
    initng_del_hook(A_FORK, &do_suid);
}
