
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef ACTIVE_STATE_H
#define ACTIVE_STATE_H

typedef struct a_state_t a_state_h;

#include "initng_list.h"
#include "initng_active_db.h"

typedef enum
{
    IS_UNKNOWN = 0,
    IS_UP = 1,
    IS_DOWN = 2,
    IS_FAILED = 3,
    IS_STARTING = 4,
    IS_STOPPING = 5,
    IS_WAITING = 6
} e_is;


struct a_state_t
{
    /* the name of the state in a string, will be printed */
    const char *state_name;

    /* If this state is set for a service, is it roghly: */
    e_is is;

    /* The handler, if set will be executed if g.interrupt is set */
    void (*state_handler) (active_h * service);

    /* if no handler is set, initng will try mark service this statys on every g.interrupt */
    a_state_h *next_state;


    /* The list this struct is in */
    struct list_head list;
};

int initng_add_state(a_state_h * state);

#define initng_del_state(st) list_del(&(st)->list)

a_state_h *initng_find_state(const char *state_name);

#define while_states(current) list_for_each_entry_prev(current, &g.states.list, list)
#define while_states_safe(current) list_for_each_entry_prev_safe(current, safe, &g.states.list, list)

#define IS_MARK(serv, state) (serv && serv->a_state && serv->a_state == state)
#define IS_UP(serv) (serv && serv->a_state && serv->a_state->is == IS_UP)
#define IS_DOWN(serv) (!serv->a_state || (serv && serv->a_state && serv->a_state->is == IS_DOWN))
#define IS_FAILED(serv) (serv && serv->a_state && serv->a_state->is == IS_FAILED)
#define IS_STARTING(serv) (serv && serv->a_state && serv->a_state->is == IS_STARTING)
#define IS_STOPPING(serv) (serv && serv->a_state && serv->a_state->is == IS_STOPPING)
#define IS_WAITING(serv) (serv && serv->a_state && serv->a_state->is == IS_WAITING)

#endif
