
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* common standard first define */
#include "initng.h"

/* system headers */
#include <time.h>                           /* time() */
#include <fcntl.h>                          /* fcntl() */
#include <sys/un.h>                         /* memmove() strcmp() */
#include <sys/wait.h>                       /* waitpid() sa */
#include <linux/kd.h>                       /* KDSIGACCEPT */
#include <sys/ioctl.h>                      /* ioctl() */
#include <stdio.h>                          /* printf() */
#include <stdlib.h>                         /* free() exit() */
#include <sys/reboot.h>                     /* reboot() RB_DISABLE_CAD */
#include <assert.h>

/* own header */
#include "initng_env_variable.h"

/* local headers include */
#include "initng_main.h"
#include "initng_active_db.h"
#include "initng_load_module.h"
#include "initng_plugin_callers.h"
#include "initng_common.h"
#include "initng_toolbox.h"
#include "initng_string_tools.h"
#include "initng_global.h"

const char *initng_environ[] = {
    "INITNG=" INITNG_VERSION,
    "INITNG_CREATOR=" INITNG_CREATOR,
    "INIT_VERSION=" INITNG_VERSION,
    "PATH=/bin:/usr/bin:/sbin:/usr/sbin:/usr/local/bin:/opt/bin",
    "HOME=/root",
    "USER=root",
    "TERM=linux",
    NULL
};


/* creates a custom environment variable */
char **new_environ(active_h * s)
{
    int allocate;
    int nr = 0;
    int i;
    char **env;

    /* Better safe than sorry... */
    if (s && s->name == NULL)
        s->name = i_strdup("unknown");


    /*
     * FIRST, try to figure out how big array we want to create.
     */

    /*  At least 11 allocatons below, and place for plugin added to (about 100) */
    allocate = 112;

    /* count existing env's */
#ifdef USE_OLD_ENV
    while (environ[nr])
        allocate++;
#endif

    /* count ENVs in service ENV variable */
    if (s)
        allocate += active_db_count_type(&ENV, s);


    /* finally allocate */
    env = (char **) initng_calloc(allocate, sizeof(char *));

    /* duplicate */
#ifdef USE_OLD_ENV
    for (nr = 0; environ[nr] && nr < allocate; nr++)
    {
        env[nr] = i_strdup(environ[nr]);
    }
#endif

    /* add all static defined abow in initng_environ */
    for (i = 0; initng_environ[i]; i++)
    {
        env[nr++] = i_strdup(initng_environ[i]);
    }


    if (s && (nr + 4) < allocate)
    {
        env[nr] =
            (char *) initng_calloc(1, sizeof(char) * (9 + strlen(s->name)));
        strcpy(env[nr], "SERVICE=");
        strcat(env[nr], s->name);
        nr++;

        env[nr] =
            (char *) initng_calloc(1, sizeof(char) * (6 + strlen(s->name)));
        strcpy(env[nr], "NAME=");
        strcat(env[nr], st_strip_path(s->name));
        nr++;

        env[nr] =
            (char *) initng_calloc(1, sizeof(char) * (7 + strlen(s->name)));
        strcpy(env[nr], "CLASS=");
        strcat(env[nr], s->name);
        st_strip_end(&env[nr]);
        nr++;

        if (g.dev_console)
        {
            env[nr] =
                (char *) initng_calloc(1,
                                       sizeof(char) * (9 +
                                                       strlen(g.
                                                              dev_console)));
            strcpy(env[nr], "CONSOLE=");
            strcat(env[nr], g.dev_console);
            nr++;
        }
        else
        {
            env[nr] =
                (char *) initng_calloc(1,
                                       sizeof(char) * (9 +
                                                       strlen
                                                       (INITNG_CONSOLE)));
            strcpy(env[nr], "CONSOLE=");
            strcat(env[nr], INITNG_CONSOLE);
            nr++;
        }

        if (g.runlevel)
        {
            env[nr] =
                (char *) initng_calloc(1,
                                       sizeof(char) * (10 +
                                                       strlen(g.runlevel)));
            strcpy(env[nr], "RUNLEVEL=");
            strcat(env[nr], g.runlevel);
            nr++;
        }

        if (g.old_runlevel)
        {
            env[nr] =
                (char *) initng_calloc(1,
                                       sizeof(char) * (14 +
                                                       strlen(g.
                                                              old_runlevel)));
            strcpy(env[nr], "PREVLEVEL=");
            strcat(env[nr], g.old_runlevel);
            nr++;
        }

        env[nr] = NULL;
        /* insert all env strings from config */
        if ((env[nr] = active_db_get_next_string(&ENV, s, NULL)))
        {
            nr++;
            while (((nr + 1) < allocate) &&
                   (env[nr] =
                    active_db_get_next_string(&ENV, s, env[nr - 1])))
                nr++;

        }
    }

    /* null last */
    if (env[nr] != NULL)
        env[nr++] = NULL;

#ifdef DEBUG
    for (nr = 0; env[nr]; nr++)
    {
        D_("environ[%i] = \"%s\"\n", nr, env[nr]);
    }
#endif

    /* return new environ */
    return (env);
}

/* this frees an environment variable */
void free_environ(char **tf)
{
    int i = 0;

    D_("free_environ();\n");
    assert(tf);
    for (i = 0; tf[i]; tf++)
    {
        free(tf[i]);
    }
    free(tf);
}


/*
 * This function will change variables in from, like $NAME, $SERVICE, $CLASS
 * into its right content, and return a new allocated string with the result.
 */
char *fix_variables(char *from, active_h * s)
{
    char *tmp;                  /* tempoaray space to use */
    char *to;                   /* this is the pointer we are going to return */
    char *set;                  /* pointer to step */
    int i;                      /* temporary counter */

    assert(s);
    assert(s->name);

    /* make sure we got any data */
    if (!from)
        return (NULL);

    /* alocate that mutch memory that we think we need */
    /* TODO, add checkings that this value is never overridden */
    to = (char *) initng_calloc((strlen(from) + 150), sizeof(char));
    if (!to)
        return (NULL);

    set = to;
    /* while there is data to parse */
    while (from[0])
    {
        int len = 0;

        /*
         * if its not containing the magical char loop, youst copy the
         * char and continue the main loop.
         */
        if (from[0] != '$')
            goto copy_char;

        /* jump the '$' char */
        from++;
        if (!from[0])
            continue;

        /*
         * Calculate lenght of variable ${VARIABLE}.
         */
        if (from[0] == '{')                 /* if this is a ${VARIABLE} */
        {
            from++;
            while (from[len] && from[len] != '}')
                len++;
        }
        else                                /* else a $VARIABLE */
        {
            while (from[len] && from[len] != ' ' && from[len] != '\t'
                   && from[len] != '"' && from[len] != '\''
                   && from[len] != '.' && from[len] != '/'
                   && from[len] != '\n')
                len++;
        }

        /*
         * Start comparing keywords.
         */

        if (strncasecmp(from, "NAME", len) == 0)
        {
            if ((tmp = st_strip_path(s->name)))
            {
                strcpy(set, tmp);
                goto go_forward;
            }
        }

        if (strncasecmp(from, "SERVICE", len) == 0)
        {
            strcpy(set, s->name);
            goto go_forward;
        }

        if (strncasecmp(from, "CLASS", len) == 0)
        {
            if ((tmp = st_get_path(s->name)))
            {
                strcpy(set, tmp);
                free(tmp);
            }
            goto go_forward;
        }

        if (g.runlevel && strncasecmp(from, "CONSOLE", len) == 0)
        {
            if (g.dev_console)
                strcpy(set, g.dev_console);
            else
                strcpy(set, INITNG_CONSOLE);

            goto go_forward;
        }

        if (g.runlevel && strncasecmp(from, "RUNLEVEL", len) == 0)
        {
            strcpy(set, g.runlevel);
            goto go_forward;
        }

        if (g.runlevel && strncasecmp(from, "PREVLEVEL", len) == 0)
        {
            strcpy(set, g.old_runlevel);
            goto go_forward;
        }

        /* Fetch variable from initng_environ */
        for (i = 0; initng_environ[i]; i++)
        {
            if (strncasecmp(from, initng_environ[i], len) == 0
                && initng_environ[i][len] == '=')
            {
                strcpy(set, &initng_environ[i][len + 1]);
                goto go_forward;
            }
        }

        /* get variable from ENV type */
        tmp = NULL;
        while ((tmp = active_db_get_next_string(&ENV, s, tmp)))
        {
            if (strncasecmp(from, tmp, len) == 0 && tmp[len] == '=')
            {
                strcpy(set, &tmp[len + 1]);
                goto go_forward;
            }
        }

        /* Get the variable from the data set on an initng_variable */
        {
            s_entry *entry = NULL;

            /* Copy the string, so that we can put an '\0' on the end */
            tmp = i_strndup(from, len);
            if (!tmp)
                goto print_error;

            /* search if a initng variable exits with that name */
            entry = initng_sdt_find(tmp);

            /* free the duped one */
            free(tmp);

            /* if we got an entry, and if the entry exits in this service */
            if (entry && active_db_is(entry, s))
            {
                /* Handle diffrently with diffrent types */
                switch (entry->opt_type)
                {
                        /* if it is a string, youst strcpy in it */
                    case STRING:
                        {
                            strcpy(set, active_db_get_string(entry, s));
                            goto go_forward;
                        }
                    case STRINGS:
                        {
                            tmp = NULL;
                            while ((tmp =
                                    active_db_get_next_string(entry, s, tmp)))
                            {
                                strcat(set, tmp);
                                strcat(set, " ");
                            }
                            if (tmp)
                                goto go_forward;
                        }
                        /* if it is an integer, sprintf in it */
                    case INT:
                        {
                            sprintf(set, "%i", active_db_get_int(entry, s));
                            goto go_forward;
                        }
                    default:
                        {
                            D_("Dont know how to set a variable with %s:%i\n",
                               entry->opt_name, entry->opt_type);
                            break;
                        }
                }
            }
        }


      print_error:
        /* Print a fancy error message */
        tmp = strndup(from, len);
        W_("Service %s Unknown variable ${%s}!\n", s->name, tmp);
        free(tmp);

      copy_char:
        /* Copy the char, and move on. */
        set[0] = from[0];
        from++;
        set++;
        set[0] = '\0';
        continue;

      go_forward:
        /* go forward */
        from += len;
        if (from[0] == '}')
            from++;
        while (set[0])
            set++;

    }

    /* make sure end is terminated */
    set[0] = '\0';

    /* return the string we now fixed */
    return (to);
}
