
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "initng.h"
#include "initng_global.h"

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "initng_struct_data.h"
#include "initng_static_data_id.h"
#include "initng_toolbox.h"

#undef D_
#define D_(fmt, ...)

static void d_free(s_data * current);

/* gives an error message and return if type is not set */
#define RETURN_TYPE_CHECK(ret) \
    /* check so that data entry exists */ \
    if (!type) \
    { \
        F_("Type cant be zero!\n"); \
        return (ret); \
    }

/* if this is an alias variable type, walk to find the correct one */
#define ALIAS_WALK \
    /* this might be an alias */ \
    while (type->opt_type == ALIAS && type->alias) \
        type = type->alias;

/* check if vn exitsts, that its correct */
#define VN_MATCH(vnc, vnt) \
    (!vnc || (vnt && strcasecmp(vnc, vnt)==0))

#define IT(x) (type->opt_type == x || type->opt_type == (x + 50))


char *d_get_string_var(s_entry * type, const char *vn,
                       struct list_head *address)
{
    s_data *current = NULL;

    assert(address);

    RETURN_TYPE_CHECK(NULL);

    ALIAS_WALK;

    if (!vn && type->opt_type >= 50)
    {
        F_("The vn variable is missing for a type %i %s!\n", type->opt_type,
           type->opt_name);
        return (NULL);
    }

    if (!IT(STRING))
    {
        F_("Cand only fetch a type, that is a STRING\n");
        return (NULL);
    }

    /* walk thru all entrys, on adress */
    list_for_each_entry(current, address, list)
    {
        /* if this type is right, return string */
        if (current->type == type && current->t.s
            && VN_MATCH(current->vn, vn))
        {
            D_("got %s\n", current->t.s);
            return (current->t.s);
        }
    }

    /* did not get any match */
    D_("String type %s doesn't exist.\n", type->opt_name);
    return (NULL);
}

int d_get_int_var(s_entry * type, const char *vn, struct list_head *d)
{
    s_data *current = NULL;

    assert(d);

    RETURN_TYPE_CHECK(0);

    ALIAS_WALK;

    if (!vn && type->opt_type >= 50)
    {
        F_("The vn variable is missing for a type %i %s!\n", type->opt_type,
           type->opt_name);
        return (0);
    }

    if (!IT(INT))
    {
        F_("Cand only fetch a type, that is a INT!\n");
        return (0);
    }

    /* walk thru all entrys on adress */
    list_for_each_entry(current, d, list)
    {
        /* if this type is right, return int */
        if (current->type == type && current->t.i
            && VN_MATCH(current->vn, vn))
            return (current->t.i);

    }

    /* did not get any match */
    D_("Int value, type %s doesn't exist in db.\n", type->opt_name);
    return (0);
}


char *d_get_next_string_var(s_entry * type, const char *vn,
                            struct list_head *d, char *last)
{
    s_data *current = NULL;

    assert(d);

    RETURN_TYPE_CHECK(NULL);

    ALIAS_WALK;

    if (!vn && type->opt_type >= 50)
    {
        F_("The vn variable is missing!\n");
        return (NULL);
    }

    if (!IT(STRINGS))
    {
        F_("Cand only fetch a type, that is a STRINGS!\n");
        return (NULL);
    }

    /* walk thru all in d, will be but into current */
    list_for_each_entry_prev(current, d, list)
    {
        /* if the returned type is not that type we are looking for, continue */
        if (current->type != type)
            continue;

        /* check that variable name if any matches */
        if (!VN_MATCH(current->vn, vn))
            continue;

        if (current->t.s == NULL)
            return (NULL);

        /* if this is the one, we start looking after, reset last, and continue */
        if (last)
        {
            if (current->t.s == last)
            {
                last = NULL;
            }
            continue;
        }

        /* if this is last, return next string */
        return (current->t.s);
    }
    /* no match */
    return (NULL);
}

/* this has to be only one of */
void d_set_string_var(s_entry * type, char *vn, struct list_head *d,
                      char *string)
{
    s_data *current = NULL;

    assert(d);
    assert(string);

    if (!type)
    {
        F_("Type cant be zero!\n");
        return;
    }

    ALIAS_WALK;

    if (!vn && type->opt_type >= 50)
    {
        F_("The vn variable is missing for a type %i %s, trying to set \"%s\"!\n", type->opt_type, type->opt_name, string);
        return;
    }


    if (!IT(STRING))
    {
        F_(" \"%s\" is not an STRING type, sleeping 1 sek ..",
           type->opt_name);
        sleep(1);
        return;
    }

    /* check the db, for an current entry to overwrite */
    list_for_each_entry(current, d, list)
    {
        if (current->type == type && VN_MATCH(current->vn, vn))
        {
            if (current->t.s)
                free(current->t.s);
            current->t.s = string;
            return;
        }
    }

    current = (s_data *) initng_calloc(1, sizeof(s_data));
    current->type = type;
    current->t.s = string;
    current->vn = vn;

    /* add this one */
    list_add(&(current->list), d);
}



/* this is a function to add a string, that it can exist many of */
void d_set_another_string_var(s_entry * type, char *vn, struct list_head *d,
                              char *string)
{
    s_data *current = NULL;

    assert(d);
    assert(string);

    if (!type)
    {
        F_("Type cant be zero!\n");
        return;
    }

    ALIAS_WALK;

    if (!vn && type->opt_type >= 50)
    {
        F_("The vn variable is missing for a type %i %s, trying to set string: \"%s\"!\n", type->opt_type, type->opt_name, string);
        return;
    }

    if (!IT(STRINGS))
    {
        F_(" \"%s\" is not an strings type!\n", type->opt_name);
        return;
    }

    current = (s_data *) initng_calloc(1, sizeof(s_data));
    current->type = type;
    current->t.s = string;
    current->vn = vn;

    /* add this one */
    list_add(&(current->list), d);
}

/* this has to be only one of */
void d_set_int_var(s_entry * type, char *vn, struct list_head *d, int value)
{
    s_data *current = NULL;

    assert(d);

    if (!type)
    {
        F_("Type cant be zero!\n");
        return;
    }

    ALIAS_WALK;

    if (!vn && type->opt_type >= 50)
    {
        F_("The vn variable is missing for a type %i %s!\n", type->opt_type,
           type->opt_name);
        return;
    }


    if (!IT(INT))
    {
        F_(" \"%s\" is not an int type!\n", type->opt_name);
        return;
    }

    /* check the db, for an current entry to overwrite */
    list_for_each_entry(current, d, list)
    {
        if (current->type == type && VN_MATCH(current->vn, vn))
        {
            current->t.i = value;
            return;
        }
    }
    current = (s_data *) initng_calloc(1, sizeof(s_data));
    current->type = type;
    current->t.i = value;
    current->vn = vn;

    /* add this one */
    list_add(&(current->list), d);
}



/* Walk through and count */
int d_count_type(s_entry * type, struct list_head *d)
{
    s_data *current = NULL;
    int count = 0;

    /* walk thru all entrys on adress */
    list_for_each_entry(current, d, list)
    {
        if (!type || current->type == type)
            count++;
    }

    return (count);
}

/* 
 * d_is can be used to check any type, if its in the db,
 * or just to check unset and set types.
 */
int d_is_var(s_entry * type, const char *vn, struct list_head *d)
{
    s_data *current = NULL;

    assert(d);

    RETURN_TYPE_CHECK(FALSE);

    ALIAS_WALK;

    if (!vn && type->opt_type >= 50)
    {
        F_("The vn variable is missing for a type %i %s!\n", type->opt_type,
           type->opt_name);
        return (FALSE);
    }

    /* walk thru all entrys on adress */
    list_for_each_entry(current, d, list)
    {
        if (current->type == type && VN_MATCH(current->vn, vn))
        {
            return (TRUE);
        }
    }

    return (FALSE);
}

void d_set_var(s_entry * type, char *vn, struct list_head *d)
{
    s_data *current = NULL;

    assert(d);

    if (!type)
    {
        F_("Type cant be zero!\n");
        return;
    }

    ALIAS_WALK;

    if (!vn && type->opt_type >= 50)
    {
        F_("The vn variable is missing for a type %i %s!\n", type->opt_type,
           type->opt_name);
        return;
    }

    if (!IT(SET))
    {
        F_("It has to be an SET type to d_set!\n");
        return;
    }

    /* check if its set alredy */
    if (IT(SET))
        if (d_is_var(type, vn, d))
            return;

    /* allocate the entry */
    current = (s_data *) initng_calloc(1, sizeof(s_data));
    current->type = type;
    current->vn = vn;

    /* add this one */
    list_add(&(current->list), d);
}

/*
 * A function to nicely free the s_data content.
 */
static void d_free(s_data * current)
{
    assert(current);
    assert(current->type);

    /* free variable data */
    switch (current->type->opt_type)
    {
        case STRING:
        case STRINGS:
        case VARIABLE_STRING:
        case VARIABLE_STRINGS:
            if (current->t.s)
                free(current->t.s);
            current->t.s = NULL;
            break;
        default:
            break;
    }

    /* free variable name, if set */
    if (current->vn)
        free(current->vn);
    current->vn = NULL;

    /* ok, free the struct */
    free(current);
}

void d_remove_var(s_entry * type, const char *vn, struct list_head *d)
{
    s_data *current, *s = NULL;

    assert(d);
    assert(type);

    if (!type)
    {
        F_("Type cant be zero!\n");
        return;
    }

    ALIAS_WALK;

    /* walk thru all entrys on adress */
    list_for_each_entry_safe(current, s, d, list)
    {
        /* if this is not the type, continue */

        if (current->type != type)
            continue;

        /* if a variable name is not specified */
        if (!vn)
        {
            D_("vn not set, freeing all.\n");
            list_del(&current->list);
            d_free(current);
            continue;
        }

        /* or if cariable name matches */
        if (VN_MATCH(current->vn, vn))
        {
            F_("vn matches, freeing\n");
            list_del(&current->list);
            d_free(current);
            continue;
        }
    }
}

/* walk through and kill all strings */
void d_remove_all(struct list_head *d)
{
    s_data *current, *s = NULL;

    assert(d);

    /* walk thru all entrys on adress */
    list_for_each_entry_safe(current, s, d, list)
    {
        /* walk, and remove all */
        list_del(&current->list);
        d_free(current);
    }
}

/*
 * Walk the db, copy all strings in list from to to.
 */
void d_copy_all(struct list_head *from, struct list_head *to)
{
    s_data *tmp = NULL;
    s_data *current = NULL;

    list_for_each_entry(current, from, list)
    {
        /* make sure type is set
           TODO, shud this be an assert(current->type) ??? */
        if (!current->type)
            continue;

        /* allocate the new one */
        tmp = (s_data *) initng_calloc(1, sizeof(s_data));

        /* copy */
        memcpy(tmp, current, sizeof(s_data));

        /* copy the data */
        switch (current->type->opt_type)
        {
            case STRING:
            case STRINGS:
            case VARIABLE_STRING:
            case VARIABLE_STRINGS:
                if (current->t.s)
                    tmp->t.s = i_strdup(current->t.s);
                break;
            default:
                break;
        }

        /* copy variable name */
        if (current->vn)
            tmp->vn = i_strdup(current->vn);
        else
            tmp->vn = NULL;

        /* add to list */
        list_add(&tmp->list, to);
    }
}
